/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.gui;

import java.awt.Desktop;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import picto.app.interfaces.Localizable;
import picto.utils.Config;
import picto.utils.Text;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gamil>
 */
public class AboutWindow extends JFrame implements Localizable{
    private static final long serialVersionUID = 7879846356384791L;

    /** Creates new form AboutWindow */
    public AboutWindow() {
        initComponents();
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        tabbedPane.setFocusable(false);
        licenseTextArea.setEditable(false);
        changeLogEditorPane.setEditable(false);
        creditsTextArea.setEditable(false);
        creditsTextArea.setCaretPosition(0);
        homepageButton.setFocusable(false);
        changeLogEditorPane.setText(loadChangeLog());
        changeLogEditorPane.setCaretPosition(0);
        licenseTextArea.setText(loadLicense());
        licenseTextArea.setCaretPosition(0);
        updateTexts();
        addWeakReference();
    }

    @Override
    public final void updateTexts() {
        setTitle(_("About..."));
        tabbedPane.setTitleAt(0, _("Presentation"));
        tabbedPane.setTitleAt(1, _("License"));
        tabbedPane.setTitleAt(2, _("Credits"));
        tabbedPane.setTitleAt(3, _("ChangeLog"));
        titleLabel.setText(Config.getAppName());
        versionLabel.setText(_("Version")+" " + Config.getAppVersion());
        descriptionLabel.setText(_(Config.getAppDescription()));
        homepageButton.setText(_("PictoGame Homepage"));
        homepageButton.setToolTipText(Config.getAppURL());
        creditsTextArea.setText(loadCredits());
        creditsTextArea.setCaretPosition(0);
    }

    @Override
    public final void addWeakReference() {
        Text.addWeakReference(this);
    }

    public static String loadCredits(){
        String credits = "";
        String temp;
        try {
            BufferedReader reader = new BufferedReader(new InputStreamReader(
                            AboutWindow.class.getResourceAsStream("/AUTHORS")));
            do{
                temp = reader.readLine();
                if (temp == null){
                } else if(temp.contains("Developers:")){
                    credits += "  "+_("Developers:")+"\n";
                } else if (temp.contains("Translators:")){
                    credits += "  "+_("Translators:")+"\n";
                } else if(temp.contains("Bug reporters:")){
                    credits += "  "+_("Bug reporters:")+"\n";
                } else if(temp.contains("Special thanks to:")){
                    credits += "  "+_("Special thanks to:")+"\n";
                } else{
                    credits += temp +"\n";
                }
            } while(temp != null);
            reader.close();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        }
        return credits;
    }

    public static String loadChangeLog(){
        String changeLog = "";
        String temp;
        try {
            BufferedReader reader = new BufferedReader(new InputStreamReader(
                        AboutWindow.class.getResourceAsStream("/ChangeLog")));
            temp = reader.readLine();
            while(temp != null){
                changeLog += temp +"\n";
                temp = reader.readLine();
            }
            reader.close();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        }
        return changeLog;
    }

    private static final int GPL_LENGHT = 0x7ED8;
    private static String loadLicense(){
        StringBuilder _GPL = new StringBuilder(GPL_LENGHT);
        String temp;
        try {
            BufferedReader reader = new BufferedReader(new InputStreamReader(
                          AboutWindow.class.getResourceAsStream("/COPYING")));
            temp = reader.readLine();
            while(temp != null){
                _GPL.append(temp).append("\n");
                temp = reader.readLine();
            } 
            reader.close();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        }
        if (_GPL.length() == GPL_LENGHT){
            return _GPL.toString();
        } else{
            return _("Unable to load the License file... Sorry");
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        tabbedPane = new javax.swing.JTabbedPane();
        presentationPanel = new javax.swing.JPanel();
        titleLabel = new javax.swing.JLabel();
        versionLabel = new javax.swing.JLabel();
        descriptionLabel = new javax.swing.JLabel();
        homepageButton = new javax.swing.JButton();
        licensePanel = new javax.swing.JPanel();
        licenseScrollPane = new javax.swing.JScrollPane();
        licenseTextArea = new javax.swing.JTextArea();
        creditsPanel = new javax.swing.JPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        creditsTextArea = new javax.swing.JTextPane();
        ChangeLog = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        changeLogEditorPane = new javax.swing.JTextPane();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        tabbedPane.setTabPlacement(javax.swing.JTabbedPane.BOTTOM);
        tabbedPane.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        tabbedPane.setDoubleBuffered(true);

        presentationPanel.setLayout(new java.awt.GridBagLayout());

        titleLabel.setFont(new java.awt.Font("URW Chancery L", 1, 48));
        titleLabel.setForeground(new java.awt.Color(77, 81, 126));
        titleLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        titleLabel.setText("Picto");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 0);
        presentationPanel.add(titleLabel, gridBagConstraints);

        versionLabel.setFont(new java.awt.Font("DejaVu Sans", 0, 18));
        versionLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        versionLabel.setText("Version x.x.x");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 4, 0);
        presentationPanel.add(versionLabel, gridBagConstraints);

        descriptionLabel.setFont(new java.awt.Font("DejaVu Sans", 1, 13));
        descriptionLabel.setForeground(new java.awt.Color(215, 151, 60));
        descriptionLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        descriptionLabel.setText("Description");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
        presentationPanel.add(descriptionLabel, gridBagConstraints);

        homepageButton.setText("HomePage");
        homepageButton.setBorderPainted(false);
        homepageButton.setFocusable(false);
        homepageButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                homepageButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        presentationPanel.add(homepageButton, gridBagConstraints);

        tabbedPane.addTab("Presentation", presentationPanel);

        licensePanel.setLayout(new java.awt.GridLayout(1, 0));

        licenseScrollPane.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        licenseTextArea.setColumns(8);
        licenseTextArea.setRows(5);
        licenseScrollPane.setViewportView(licenseTextArea);

        licensePanel.add(licenseScrollPane);

        tabbedPane.addTab("License", licensePanel);

        creditsPanel.setLayout(new java.awt.GridLayout(1, 1));

        jScrollPane2.setViewportView(creditsTextArea);

        creditsPanel.add(jScrollPane2);

        tabbedPane.addTab("Credits", creditsPanel);

        jScrollPane1.setViewportView(changeLogEditorPane);

        javax.swing.GroupLayout ChangeLogLayout = new javax.swing.GroupLayout(ChangeLog);
        ChangeLog.setLayout(ChangeLogLayout);
        ChangeLogLayout.setHorizontalGroup(
            ChangeLogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 473, Short.MAX_VALUE)
        );
        ChangeLogLayout.setVerticalGroup(
            ChangeLogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 227, Short.MAX_VALUE)
        );

        tabbedPane.addTab("ChangeLog", ChangeLog);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(tabbedPane, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 485, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(tabbedPane, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 270, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void homepageButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_homepageButtonActionPerformed
        if (Desktop.isDesktopSupported()) {
            Desktop desktop = Desktop.getDesktop();
            try {
                desktop.browse(new URI(Config.getAppURL()));
            } catch (URISyntaxException ex) {
                Logger.getLogger(AboutWindow.class.getName())
                        .log(Level.SEVERE, null, ex);
            } catch (IOException ex) {
                Logger.getLogger(AboutWindow.class.getName())
                        .log(Level.SEVERE, null, ex);
            }
        }
}//GEN-LAST:event_homepageButtonActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel ChangeLog;
    private javax.swing.JTextPane changeLogEditorPane;
    private javax.swing.JPanel creditsPanel;
    private javax.swing.JTextPane creditsTextArea;
    private javax.swing.JLabel descriptionLabel;
    private javax.swing.JButton homepageButton;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JPanel licensePanel;
    private javax.swing.JScrollPane licenseScrollPane;
    private javax.swing.JTextArea licenseTextArea;
    private javax.swing.JPanel presentationPanel;
    private javax.swing.JTabbedPane tabbedPane;
    private javax.swing.JLabel titleLabel;
    private javax.swing.JLabel versionLabel;
    // End of variables declaration//GEN-END:variables

}
