/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.gui;

import java.awt.Color;
import java.awt.Dialog;
import java.awt.Frame;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import picto.app.interfaces.Localizable;
import picto.utils.Config;
import picto.utils.Text;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class ColorChooser extends JDialog implements Localizable{
    private static final long serialVersionUID = 165789468537246352L;

    /** Creates new form ColorChooser */
    public ColorChooser(Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        updateTexts();
        loadColors();
        addWeakReference();
    }

    private String title = _("Select Color");
    private String s = "<html><i><b>";
    private String e = "</b></i>";
    @Override
    public final void updateTexts() {
        title = _("Select Color");
        setTitle(title);
        closeButton.setText(_("Close"));
        resetButton.setText(_("Defaults"));
        if (Config.getInstance().getColorButtonBackgroundEmpty().equals(
                Config.getDefaultInstance().getColorButtonBackgroundEmpty())){
            jLabel1.setText(_("Default cell"));
        } else {
            jLabel1.setText(s+_("Default cell")+e);
        }
        if (Config.getInstance().getColorButtonBackgroundSomething().equals(
                Config.getDefaultInstance().getColorButtonBackgroundSomething())){
            jLabel2.setText(_("Marked cell"));
        } else {
            jLabel2.setText(s+_("Marked cell")+e);
        }
        if (Config.getInstance().getColorButtonBackgroundNothing().equals(
                Config.getDefaultInstance().getColorButtonBackgroundNothing())){
            jLabel4.setText(_("Empty cell"));
        } else {
            jLabel4.setText(s+_("Empty cell")+e);
        }
        if (Config.getInstance().getColorButtonBorderColor().equals(
                Config.getDefaultInstance().getColorButtonBorderColor())){
            jLabel3.setText(_("Inner border"));
        } else {
            jLabel3.setText(s+_("Inner border")+e);
        }
        if (Config.getInstance().getHeaderBackgroundColor().equals(
                Config.getDefaultInstance().getHeaderBackgroundColor())){
            jLabel5.setText(_("Headers"));
        } else {
            jLabel5.setText(s+_("Headers")+e);
        }
        if (Config.getInstance().getHeaderBorderColor().equals(
                Config.getDefaultInstance().getHeaderBorderColor())){
            jLabel6.setText(_("Header border"));
        } else {
            jLabel6.setText(s+_("Header border")+e);
        }
        if (Config.getInstance().getHeaderBackgroundFocusColor().equals(
                Config.getDefaultInstance().getHeaderBackgroundFocusColor())){
            jLabel7.setText(_("Header focus"));
        } else {
            jLabel7.setText(s+_("Header focus")+e);
        }
        if (Config.getInstance().getBackgroundColor().equals(
                Config.getDefaultInstance().getBackgroundColor())){
            jLabel8.setText(_("Background color"));
        } else {
            jLabel8.setText(s+_("Background color")+e);
        }
        if (Config.getInstance().getStrikedTextForeground().equals(
                Config.getDefaultInstance().getStrikedTextForeground())){
            jLabel10.setText(_("Striked header color"));
        } else {
            jLabel10.setText(s+_("Striked header color")+e);
        }
        jButton1.setForeground(Config.getInstance().getHeaderFontForeground());
        jButton1.setFont(Config.getInstance().getHeaderFont());
        if (Config.getInstance().getHeaderFont().equals(
                Config.getDefaultInstance().getHeaderFont())){
            jButton1.setText(_("Header font"));
        } else {
            jButton1.setText(s+_("Header font")+e);
        }
        ((MainWindow)getParent()).updateColors();
    }

    @Override
    public final void addWeakReference() {
        Text.addWeakReference(this);
    }

    private void loadColors(){
        jButton3.setBackground(Config.getInstance().getColorButtonBackgroundEmpty());
        jButton4.setBackground(Config.getInstance().getColorButtonBackgroundSomething());
        jButton5.setBackground(Config.getInstance().getColorButtonBackgroundNothing());
        jButton6.setBackground(Config.getInstance().getColorButtonBorderColor());
        jButton7.setBackground(Config.getInstance().getHeaderBackgroundColor());
        jButton8.setBackground(Config.getInstance().getHeaderBorderColor());
        jButton9.setBackground(Config.getInstance().getHeaderBackgroundFocusColor());
        jButton10.setBackground(Config.getInstance().getBackgroundColor());
        jButton11.setBackground(Config.getInstance().getStrikedTextForeground());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        jPanel1 = new javax.swing.JPanel();
        jButton3 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();
        jButton5 = new javax.swing.JButton();
        jButton6 = new javax.swing.JButton();
        jButton7 = new javax.swing.JButton();
        jButton8 = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jButton9 = new javax.swing.JButton();
        jButton10 = new javax.swing.JButton();
        jLabel8 = new javax.swing.JLabel();
        jButton1 = new javax.swing.JButton();
        jLabel10 = new javax.swing.JLabel();
        jButton11 = new javax.swing.JButton();
        closeButton = new javax.swing.JButton();
        resetButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setAlwaysOnTop(true);

        jButton3.setBackground(new java.awt.Color(255, 51, 0));
        jButton3.setText(" ");
        jButton3.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jButton3.setContentAreaFilled(false);
        jButton3.setOpaque(true);
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        jButton4.setText(" ");
        jButton4.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jButton4.setContentAreaFilled(false);
        jButton4.setOpaque(true);
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });

        jButton5.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jButton5.setContentAreaFilled(false);
        jButton5.setOpaque(true);
        jButton5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton5ActionPerformed(evt);
            }
        });

        jButton6.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jButton6.setContentAreaFilled(false);
        jButton6.setOpaque(true);
        jButton6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton6ActionPerformed(evt);
            }
        });

        jButton7.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jButton7.setContentAreaFilled(false);
        jButton7.setOpaque(true);
        jButton7.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton7ActionPerformed(evt);
            }
        });

        jButton8.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jButton8.setContentAreaFilled(false);
        jButton8.setOpaque(true);
        jButton8.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton8ActionPerformed(evt);
            }
        });

        jLabel1.setText("Empty field");

        jLabel2.setText("Full field");

        jLabel3.setText("Inner border");

        jLabel4.setText("None field");

        jLabel5.setText("header");

        jLabel6.setText("header border");

        jLabel7.setText("background color");

        jButton9.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jButton9.setContentAreaFilled(false);
        jButton9.setOpaque(true);
        jButton9.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton9ActionPerformed(evt);
            }
        });

        jButton10.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jButton10.setContentAreaFilled(false);
        jButton10.setOpaque(true);
        jButton10.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton10ActionPerformed(evt);
            }
        });

        jLabel8.setText("background color");

        jButton1.setText("Font");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jLabel10.setText("background color");

        jButton11.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jButton11.setContentAreaFilled(false);
        jButton11.setOpaque(true);
        jButton11.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton11ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jButton3, javax.swing.GroupLayout.PREFERRED_SIZE, 28, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jButton4)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jButton5)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jButton6)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jButton7)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel5, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jButton8)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel6, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jButton9)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel7))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jButton10)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel8))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jButton11)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel10))
                    .addComponent(jButton1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel1Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jButton10, jButton11, jButton3, jButton4, jButton5, jButton6, jButton7, jButton8, jButton9});

        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton3)
                    .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton4)
                    .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton5)
                    .addComponent(jLabel4, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton6)
                    .addComponent(jLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton7)
                    .addComponent(jLabel5, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton8)
                    .addComponent(jLabel6, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton9)
                    .addComponent(jLabel7, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton10)
                    .addComponent(jLabel8, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton11)
                    .addComponent(jLabel10, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButton1)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel1Layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {jButton10, jButton11, jButton3, jButton4, jButton5, jButton6, jButton7, jButton8, jButton9, jLabel1, jLabel10, jLabel2, jLabel3, jLabel4, jLabel5, jLabel6, jLabel7, jLabel8});

        jScrollPane1.setViewportView(jPanel1);

        closeButton.setText("jButton1");
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });

        resetButton.setText("jButton2");
        resetButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resetButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap(147, Short.MAX_VALUE)
                .addComponent(resetButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(closeButton)
                .addContainerGap())
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 297, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 214, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(closeButton)
                    .addComponent(resetButton))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getInstance().getColorButtonBackgroundEmpty());
        if (temp != null){
            Config.getInstance().setColorButtonBackgroundEmpty(temp);
            jButton3.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_jButton3ActionPerformed

    private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getInstance().getColorButtonBackgroundSomething());
        if (temp != null){
            Config.getInstance().setColorButtonBackgroundSomething(temp);
            jButton4.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_jButton4ActionPerformed

    private void jButton5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton5ActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getInstance().getColorButtonBackgroundNothing());
        if (temp != null){
            Config.getInstance().setColorButtonBackgroundNothing(temp);
            jButton5.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_jButton5ActionPerformed

    private void jButton6ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton6ActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getInstance().getColorButtonBorderColor());
        if (temp != null){
            Config.getInstance().setColorButtonBorderColor(temp);
            jButton6.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_jButton6ActionPerformed

    private void jButton7ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton7ActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getInstance().getHeaderBackgroundColor());
        if (temp != null){
            Config.getInstance().setHeaderBackgroundColor(temp);
            jButton7.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_jButton7ActionPerformed

    private void jButton8ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton8ActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getInstance().getHeaderBorderColor());
        if (temp != null){
            Config.getInstance().setHeaderBorderColor(temp);
            jButton8.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_jButton8ActionPerformed

    private void jButton9ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton9ActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getInstance().getHeaderBackgroundFocusColor());
        if (temp != null){
            Config.getInstance().setHeaderBackgroundFocusColor(temp);
            jButton9.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_jButton9ActionPerformed

    private void resetButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resetButtonActionPerformed
        Config.getInstance().setColorButtonBackgroundEmpty(Config.getDefaultInstance().getColorButtonBackgroundEmpty());
        Config.getInstance().setColorButtonBackgroundSomething(Config.getDefaultInstance().getColorButtonBackgroundSomething());
        Config.getInstance().setColorButtonBackgroundNothing(Config.getDefaultInstance().getColorButtonBackgroundNothing());
        Config.getInstance().setColorButtonBorderColor(Config.getDefaultInstance().getColorButtonBorderColor());
        Config.getInstance().setHeaderBackgroundColor(Config.getDefaultInstance().getHeaderBackgroundColor());
        Config.getInstance().setHeaderBorderColor(Config.getDefaultInstance().getHeaderBorderColor());
        Config.getInstance().setHeaderBackgroundFocusColor(Config.getDefaultInstance().getHeaderBackgroundFocusColor());
        Config.getInstance().setBackgroundColor(Config.getDefaultInstance().getBackgroundColor());
        Config.getInstance().setHeaderFontForeground(Config.getDefaultInstance().getHeaderFontForeground());
        Config.getInstance().setStrikedTextForeground(Config.getDefaultInstance().getStrikedTextForeground());
        Config.getInstance().setHeaderFont(Config.getDefaultInstance().getHeaderFont());
//        Config.getInstance().resetToDefaults();
        updateTexts();
        loadColors();
    }//GEN-LAST:event_resetButtonActionPerformed

    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        setVisible(false);
    }//GEN-LAST:event_closeButtonActionPerformed

    private void jButton10ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton10ActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getInstance().getBackgroundColor());
        if (temp != null){
            Config.getInstance().setBackgroundColor(temp);
            jButton10.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_jButton10ActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        FontChooser f = new FontChooser((Dialog)this);
        f.setFontAttributesToNormal(false);
        SimpleAttributeSet a = new SimpleAttributeSet();
        StyleConstants.setFontFamily(a, Config.getInstance().getHeaderFont().getName());
        StyleConstants.setFontSize(a, Config.getInstance().getHeaderFont().getSize());
        StyleConstants.setForeground(a, Config.getInstance().getHeaderFontForeground());
        f.setAttributes(a);
        f.setVisible(true);
        if (f.getOption() == JOptionPane.OK_OPTION){
            Config.getInstance().setHeaderFontForeground(f.getForeground());
            Config.getInstance().setHeaderFont(f.getFont());
            jButton1.setForeground(Config.getInstance().getHeaderFontForeground());
            jButton1.setFont(Config.getInstance().getHeaderFont());
            ((MainWindow)getParent()).updateColors();
        }
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton11ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton11ActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getInstance().getStrikedTextForeground());
        if (temp != null){
            Config.getInstance().setStrikedTextForeground(temp);
            jButton9.setBackground(temp);
            ((MainWindow)getParent()).updateColors();
        }
    }//GEN-LAST:event_jButton11ActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton closeButton;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton10;
    private javax.swing.JButton jButton11;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton5;
    private javax.swing.JButton jButton6;
    private javax.swing.JButton jButton7;
    private javax.swing.JButton jButton8;
    private javax.swing.JButton jButton9;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton resetButton;
    // End of variables declaration//GEN-END:variables

}
