/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractListModel;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import picto.utils.Picto;
import picto.app.interfaces.Localizable;
import picto.utils.Text;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gamil>
 */
public class GameLoader extends JDialog implements Localizable{
    private static final long serialVersionUID = -63765627876372891L;
    private static int NUMBER_OF_PATTERNS = 170;
    private MainWindow father;

    /** Creates new form GameLoader */
    public GameLoader(MainWindow father) {
        this.father = father;
        setModal(true);
        initComponents();
        descriptionTextPane.setEditable(false);
        descriptionTextPane.setContentType("text/html");
        setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
        addActionListeners();
        loadList();
        updateTexts();
        addWeakReference();
    }

    public void getRandomGame(){
        father.setGame(games[new Random().nextInt(games.length)]);
    }

    @Override
    public final void updateTexts() {
        fc = new JFileChooser();
        setTitle(_("Load game"));
        openFromFileButton.setText(_("Open from file"));
        openSelectedButton.setText(_("Open"));
        setDescription();
    }

    @Override
    public final void addWeakReference() {
        Text.addWeakReference(this);
    }

    private void setDescription(){
        int idx = gameList.getSelectedIndex();
        if (idx == -1){
            return;
        }
        String temp = "<html>"
                    + "<b>"+_("Name:")  +" </b>"+games[idx].getTitle()+"<br>"
                    + "<b>"+_("Author:")+" </b>"+games[idx].getAuthor()+"<br>"
                    + "<b>"+_("Date:")  +" </b>"+games[idx].getDate()+"<br>"
                    + "<b>"+_("Size:")  +" </b>"+games[idx].getNumberOfRows()
                                +"x"+games[idx].getNumberOfColumns()+"</html>";
        descriptionTextPane.setText(temp);
    }

    private static final String path = "/res/patterns/";
    private static Picto[] games;
    private void loadList(){
        BufferedReader reader = new BufferedReader(new InputStreamReader(
                 GameLoader.class.getResourceAsStream(path + "patterns.list")));
        ArrayList<String> list = new ArrayList<String>(NUMBER_OF_PATTERNS);
        String line = null;
        String[] filenameList;
        //Load the 'patterns.list' file
        try {
            line = reader.readLine();
            while (line != null){
                line = line.trim();
                if(!line.startsWith("#") && !line.isEmpty()){
                    list.add(line);
                    line = reader.readLine();
                }
            }
            reader.close();
        } catch (IOException ex) {
            Logger.getLogger(GameLoader.class.getName()).log(Level.SEVERE, null, ex);
        }
        filenameList = list.toArray(new String[0]);
        NUMBER_OF_PATTERNS = filenameList.length;
        final String[] elements = new String[NUMBER_OF_PATTERNS];

        // Load the games that appear in the patterns list
        games = new Picto[NUMBER_OF_PATTERNS];
        for (int i = 0; i < NUMBER_OF_PATTERNS; i++){
            games[i] = new Picto(GameLoader.class.getResourceAsStream
                                   (path + filenameList[i] + ".pattern"), true);
            elements[i] = "";
            if (i < 9){
                elements[i] += "00";
            } else if (i < 99){
                elements[i] += "0";
            }
            elements[i] += (i+1)+". "+games[i].getTitle();
        }

        gameList.setModel(new AbstractListModel() {
            private static final long serialVersionUID = 19415617846756343L;
            String[] strings = elements;
            @Override public int getSize() { return strings.length; }
            @Override public Object getElementAt(int i) { return strings[i]; }
        });
        gameList.setSelectedIndex(0);
    }

    private JFileChooser fc;
    private void addActionListeners(){
        openFromFileButton.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                fc.setDialogTitle(_("Open pattern"));
                fc.setDialogType(JFileChooser.OPEN_DIALOG);
                fc.setMultiSelectionEnabled(false);
                fc.setFileHidingEnabled(true);
                fc.setFileFilter(new FileNameExtensionFilter(
                                        _("PictoGame Pattern file"), "pattern"));
                if (fc.showOpenDialog(father) == JFileChooser.APPROVE_OPTION){
                    try{
                        Picto temp = new Picto(new BufferedReader(
                                        new FileReader(fc.getSelectedFile())));
                        father.setGame(temp);
                        setVisible(false);
                    } catch (Exception ex){
                        JOptionPane.showMessageDialog(null, _("Error opening file!"),
                                        _("Oops..."), JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        });
        openSelectedButton.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                father.setGame(games[gameList.getSelectedIndex()]);
                setVisible(false);
            }
        });
        gameList.addListSelectionListener(new ListSelectionListener() {
            @Override public void valueChanged(ListSelectionEvent evt) {
                setDescription();
            }
        });
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        gameList = new javax.swing.JList();
        openSelectedButton = new javax.swing.JButton();
        openFromFileButton = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        descriptionTextPane = new javax.swing.JTextPane();

        gameList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jScrollPane1.setViewportView(gameList);

        openSelectedButton.setText("Open");

        openFromFileButton.setText("Open from file");

        descriptionTextPane.setFont(new java.awt.Font("DejaVu Sans", 0, 14));
        jScrollPane2.setViewportView(descriptionTextPane);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 245, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 241, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(openFromFileButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(openSelectedButton)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 250, Short.MAX_VALUE)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 250, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(openSelectedButton)
                    .addComponent(openFromFileButton))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextPane descriptionTextPane;
    private javax.swing.JList gameList;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JButton openFromFileButton;
    private javax.swing.JButton openSelectedButton;
    // End of variables declaration//GEN-END:variables

}
