/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received generateList copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.gui;

import javax.swing.JFrame;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import picto.app.interfaces.Localizable;
import picto.utils.Text;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class HelpWindow extends JFrame implements Localizable, TreeSelectionListener{
    private static final long serialVersionUID = 34569874903975803L;

    /** Creates new form HelpWindow */
    public HelpWindow() {
        initComponents();
        generateList();
        updateTexts();
        addWeakReference();
        setSize(500, 300);
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        editorPane.setContentType("text/html");
        jSplitPane1.setDividerLocation(200);
    }

    @Override
    public final void updateTexts() {
        setTitle(_("Help"));
        indexTree.revalidate();
        indexTree.updateUI();
        if (ev != null){
            valueChanged(ev);
        }
    }
    
    private void generateList(){
        DefaultMutableTreeNode treeNode1 = new DefaultMutableTreeNode(new Help("Help", null));
        DefaultMutableTreeNode treeNode2 = new DefaultMutableTreeNode(new Help("Introduction", "IOU an intro"));
        treeNode1.add(treeNode2);
        treeNode2 = new DefaultMutableTreeNode(new Help("Menus", null));
        DefaultMutableTreeNode treeNode3 = new DefaultMutableTreeNode(new Help("Game", null));
        DefaultMutableTreeNode treeNode4 = new DefaultMutableTreeNode(new Help("Open random phrase", "OPEN_RAND_PHRASE"));
        treeNode3.add(treeNode4);
        treeNode4 = new DefaultMutableTreeNode(new Help("Open random pattern", "OPEN_RAND_GAME"));
        treeNode3.add(treeNode4);
        treeNode4 = new DefaultMutableTreeNode(new Help("Open pattern", "OPEN_PATTERN"));
        treeNode3.add(treeNode4);
        treeNode4 = new DefaultMutableTreeNode(new Help("Open...", "OPEN_MI"));
        treeNode3.add(treeNode4);
        treeNode4 = new DefaultMutableTreeNode(new Help("Save...", "SAVE_MI"));
        treeNode3.add(treeNode4);
        treeNode4 = new DefaultMutableTreeNode(new Help("Quit", "QUIT_MI"));
        treeNode3.add(treeNode4);
        treeNode2.add(treeNode3);
        treeNode3 = new DefaultMutableTreeNode(new Help("Edit", null));
        treeNode4 = new DefaultMutableTreeNode(new Help("Locale", "LOCALE_MI"));
        treeNode3.add(treeNode4);
        treeNode4 = new DefaultMutableTreeNode(new Help("Enable dragging", "ENABLE_DRAGGING"));
        treeNode3.add(treeNode4);
        treeNode4 = new DefaultMutableTreeNode(new Help("Enable mouse tracking", "ENABLE_MOUSE_TR"));
        treeNode3.add(treeNode4);
        treeNode4 = new DefaultMutableTreeNode(new Help("Colors", "COLORS_MI"));
        treeNode3.add(treeNode4);
        treeNode2.add(treeNode3);
        treeNode3 = new DefaultMutableTreeNode(new Help("Pattern", null));
        treeNode4 = new DefaultMutableTreeNode(new Help("Clear board", "CLEAR_BOARD"));
        treeNode3.add(treeNode4);
        treeNode4 = new DefaultMutableTreeNode(new Help("Check board", "CHECK_BOARD"));
        treeNode3.add(treeNode4);
        treeNode4 = new DefaultMutableTreeNode(new Help("Custom phrase", "CUSTOM_PHRASE"));
        treeNode3.add(treeNode4);
        treeNode4 = new DefaultMutableTreeNode(new Help("New pattern", "NEW_PATTERN"));
        treeNode3.add(treeNode4);
        treeNode2.add(treeNode3);
        treeNode3 = new DefaultMutableTreeNode(new Help("Extras...", "Have you ever played Contra?"));
        treeNode2.add(treeNode3);
        treeNode1.add(treeNode2);
        indexTree.setModel(new DefaultTreeModel(treeNode1));
        indexTree.setRootVisible(false);
        indexTree.addTreeSelectionListener(this);
    }

    private TreeSelectionEvent ev;
    @Override
    public void valueChanged(TreeSelectionEvent e) {
        ev = e;
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           indexTree.getLastSelectedPathComponent();
        if (node == null || !node.isLeaf()){return;}
        String foo = "<html><b>"
                   + getTitle(e.getNewLeadSelectionPath().toString())
                   + "</b><br><br><br>"
                   + ((Help)node.getUserObject()).getContent().replace("\\n", "<br>")
                   + "</html>";

        editorPane.setText(foo);
    }

    private String getTitle(String path){
        path = path.toUpperCase();
        path = path.replace(", ", " -> ");
        return path;
    }
    
    private class Help{
        private String title;
        private String content;

        public Help(String t, String c){
            title = t;
            content = c;
        }

        public String getContent(){
            return _(content);
        }

        @Override public String toString(){
            return _(title);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSplitPane1 = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        indexTree = new javax.swing.JTree();
        jScrollPane2 = new javax.swing.JScrollPane();
        editorPane = new javax.swing.JTextPane();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        jSplitPane1.setDividerLocation(90);

        javax.swing.tree.DefaultMutableTreeNode treeNode1 = new javax.swing.tree.DefaultMutableTreeNode("root");
        indexTree.setModel(new javax.swing.tree.DefaultTreeModel(treeNode1));
        indexTree.setRootVisible(false);
        jScrollPane1.setViewportView(indexTree);

        jSplitPane1.setLeftComponent(jScrollPane1);

        jScrollPane2.setDoubleBuffered(true);
        jScrollPane2.setViewportView(editorPane);

        jSplitPane1.setRightComponent(jScrollPane2);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 297, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 184, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            @Override public void run() {
                new HelpWindow().setVisible(true);
            }
        });
    }

    @Override
    public final void addWeakReference() {
        Text.addWeakReference(this);
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextPane editorPane;
    private javax.swing.JTree indexTree;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSplitPane jSplitPane1;
    // End of variables declaration//GEN-END:variables

}
