/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received generateList copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.gui;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;

import java.util.Calendar;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileNameExtensionFilter;

import picto.utils.Picto;
import picto.app.interfaces.Locales;
import picto.app.interfaces.Localizable;
import picto.app.interfaces.Playable;
import picto.utils.ColoredButton;
import picto.utils.Config;
import picto.utils.PatternDataSetter;
import picto.utils.PhraseScrambler;
import picto.utils.Saver;
import picto.utils.SavingUtils;
import picto.utils.Text;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gamil>
 */
public class MainWindow extends JFrame implements Localizable{
    private static final long serialVersionUID = -765348566788957685L;
    private int xSize, ySize;
    private Picto pic;
    private JPanel board;

    /** Creates new form MainWindow */
    public MainWindow() {
        initComponents();
        generateMenues();
        addWeakReference();
        updateColors();
        updateTexts();
        updateMenues();
        addActionListeners();
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setIconImage(new ImageIcon(GameLoader.class
                            .getResource("/res/images/icon.png")).getImage());
    }

    @Override
    public final void updateTexts() {
        appMenu.setText(_("Game"));
        appMenu.setMnemonic('G');
        openRandomPhraseMenuItem.setText(_("Open random phrase"));
        openRandomPhraseMenuItem.setMnemonic('s');
        openRandomGameMenuItem.setText(_("Open random pattern"));
        openPatternMenuItem.setText(_("Open pattern"));
        loadGameMenuItem.setText(_("Open..."));
        loadGameMenuItem.setToolTipText(_("Open saved game"));
        saveGameMenuItem.setText(_("Save..."));
        saveGameMenuItem.setToolTipText(_("Save current game"));
        quitMenuItem.setText(_("Quit"));
        editMenu.setText(_("Edit"));
        editMenu.setMnemonic('d');
        localeMenu.setText(_("Locale"));
        enMenuItem.setText(_("English"));
        esMenuItem.setText(_("Spanish"));
        colorsMenuItem.setText(_("Colors"));
        dragMenuItem.setText(_("Enable dragging"));
        mouseTrackerMenuItem.setText(_("Enable mouse tracking"));
        patternMenu.setText(_("Pattern"));
        patternMenu.setMnemonic('t');
        clearMenuItem.setText(_("Clear board"));
        checkMenuItem.setText(_("Check board"));
        checkMenuItem.setToolTipText(_("Check if some cell is wrongly marked"));
        newPhraseMenuItem.setText(_("Custom phrase"));
        newPatternMenuItem.setText(_("New pattern"));
        extrasMenu.setText(_("Extras..."));
        extrasMenu.setMnemonic('x');
        checkAndResetMenuItem.setText(_("Check board and reset wrong"));
        checkAndResetMenuItem.setToolTipText(_("Check if some cell is wrongly marked, and resets it"));
        solveMenuItem.setText(_("Show solution"));
        helpMenu.setMnemonic('?');
        helpMenuItem.setText(_("Help"));
        aboutMenuItem.setText(_("About..."));
        setTitle();
        switch (Config.getInstance().getLocale()){
            case Locales.ENGLISH:
                enMenuItem.doClick();
                break;
            case Locales.SPANISH:
                esMenuItem.doClick();
                break;
        }
    }

    @Override
    public final void addWeakReference() {
        Text.addWeakReference(this);
    }

    public void setGame(Picto pic){
        if (this.pic != null){
            this.pic.flush();
        }
        this.pic = pic;
        this.pic.regenerate();
        if (board != null){
            jPanel1.remove(board);
        }
        board = new PlayingBoard(pic);
        addPanel();
    }

    private void addPanel(){
        xSize = ((Playable)board).getXSize();
        ySize = ((Playable)board).getYSize();
        board.setVisible(true);
        jPanel1.add(board);
        centerPanel();
        jPanel1.updateUI();
    }

    private static String ERROR_MSG2 = _("File doesn't appear to be\na valid PictoSaveFile");
    /**
     * Saves the current pattern
     */
    private void saveGame(){
        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle(_("Select file to save"));
        fc.setFileHidingEnabled(true);
        fc.setMultiSelectionEnabled(false);
        if (board instanceof PlayingBoard){// Playing
            fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
            fc.setFileFilter(new FileNameExtensionFilter(_("PictoGame save file"), "PictoSave"));
            fc.setSelectedFile(new File (Calendar.getInstance().getTime().getTime() + ".PictoSave"));
            if (fc.showSaveDialog(null) == JFileChooser.APPROVE_OPTION){
                int response = JOptionPane.YES_OPTION;
                if (fc.getSelectedFile().exists()){
                    response = JOptionPane.showConfirmDialog (null,
                           _("Overwrite existing file?"),_("Confirm overwrite"),
                            JOptionPane.OK_CANCEL_OPTION,
                            JOptionPane.QUESTION_MESSAGE);
                }
                if (response == JOptionPane.YES_OPTION){
                    SavingUtils.saveGame(fc.getSelectedFile(), ((PlayingBoard)board).getSaver());
                }
            }
        } else if (board instanceof CreatePatternBoard){// Save generateList new pattern
            fc.setFileFilter(new FileNameExtensionFilter(_("PictoGame pattern file"), "pattern"));
            fc.setSelectedFile(new File (_("my_pattern")+".pattern"));
            if (fc.showSaveDialog(null) == JFileChooser.APPROVE_OPTION){
                SavingUtils.saveNewPattern(fc.getSelectedFile()
                                        , ((CreatePatternBoard)board).getPic());
            }
        }
    }

    private void loadGame(){
        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle(_("Select file to load"));
        fc.setFileHidingEnabled(true);
        fc.setMultiSelectionEnabled(false);
        fc.setFileFilter(new FileNameExtensionFilter(_("PictoGame save file"), "PictoSave"));
        if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION){
            File f = fc.getSelectedFile();
            try {
                ObjectInputStream ois = new ObjectInputStream(new FileInputStream(f));
                if (board != null){
                    jPanel1.remove(board);
                }
                Saver temp = (Saver)ois.readObject();
                ois.close();
                board = new PlayingBoard(temp);
                if (pic != null){
                    pic.flush();
                    pic = null;
                }
                if (temp.getPs() == null){
                    pic = temp.getPic();
                } 
                xSize = ((Playable)board).getXSize();
                ySize = ((Playable)board).getYSize();
                board.setVisible(true);
                jPanel1.add(board);
                centerPanel();
                jPanel1.updateUI();
            } catch (IOException ex) {
                Logger.getLogger(MainWindow.class.getName()).log(Level.SEVERE, null, ex);
            } catch (Exception ex){
                JOptionPane.showMessageDialog(null, _(ERROR_MSG2)
                                        , _("Error"), JOptionPane.ERROR_MESSAGE);
                Logger.getLogger(MainWindow.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    /**
     * Updates all the colors after generateList change in the Colors window
     * @see picto.app.gui.ColorChooser
     */
    public final void updateColors(){
        if (board != null){((Playable)board).updateColors();}
        jPanel1.setBackground(Config.getInstance().getBackgroundColor());
    }

    /**
     * Sets the title according to the situation
     */
    private void setTitle(){
        if (pic == null){
            setTitle("PictoGame v"+Config.getAppVersion());
        } else if (pic.getAuthor().isEmpty()){
            setTitle("PictoGame v"+Config.getAppVersion()
                    +" ["+pic.getTitle()+"]");
        } else {
            setTitle("PictoGame v"+Config.getAppVersion()
                    +" ["+pic.getTitle()+" - "+pic.getAuthor()+"]");
        }
        updateMenues();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new ImagePanel(null);

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 713, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 430, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    // End of variables declaration//GEN-END:variables

    /**
     * Centers the game panel to ensure that the cells are always square
     */
    private void centerPanel() {
        if (xSize == 0 || ySize == 0){return;}
        int w = jPanel1.getWidth(), h = jPanel1.getHeight();
        int newW = (w / xSize * ySize > h) ? h / ySize : w / xSize;
        board.setLocation((w -(newW * xSize)) / 2, (h -(newW * ySize)) / 2);
        board.setSize(newW * xSize, newW * ySize);
        board.validate();
    }

    /**
     * Generates the main menu
     */
    private void generateMenues(){
        menuBar.add(appMenu);
        appMenu.add(openRandomPhraseMenuItem);
        appMenu.add(openRandomGameMenuItem);
        appMenu.add(openPatternMenuItem);
        appMenu.addSeparator();
        appMenu.add(loadGameMenuItem);
        appMenu.add(saveGameMenuItem);
        appMenu.addSeparator();
        appMenu.add(quitMenuItem);
        menuBar.add(editMenu);
        editMenu.add(localeMenu);
        localeMenu.add(enMenuItem);
        localeMenu.add(esMenuItem);
        localeButtonGroup.add(enMenuItem);
        localeButtonGroup.add(esMenuItem);
        editMenu.add(dragMenuItem);
        editMenu.add(mouseTrackerMenuItem);
        editMenu.add(colorsMenuItem);
        menuBar.add(patternMenu);
        patternMenu.add(clearMenuItem);
        patternMenu.add(checkMenuItem);
        patternMenu.add(newPhraseMenuItem);
        patternMenu.add(newPatternMenuItem);
        menuBar.add(helpMenu);
        helpMenu.add(helpMenuItem);
        helpMenu.add(aboutMenuItem);
        setJMenuBar(menuBar);
    }

    /**
     * Adds all the action listeners
     */
    private void addActionListeners(){
        openRandomPhraseMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                if (board != null){
                    jPanel1.remove(board);
                }
                if (pic != null){
                    pic.flush();
                    pic = null;
                }
                board = new PlayingBoard(PhraseScrambler.getRandomScrambler());
                addPanel();
                setTitle();
                updateMenues();
            }
        });
        openRandomGameMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                gameLoader.getRandomGame();
                setTitle();
                updateMenues();
            }
        });
        openPatternMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                gameLoader.setVisible(true);
                setTitle();
                updateMenues();
            }
        });
        newPhraseMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                String foo = JOptionPane.showInputDialog(MainWindow.this
                        , _("Enter phrase"), _("Custom phrase")
                        , JOptionPane.OK_CANCEL_OPTION);
                if (foo != null){
                    if (board != null){
                        jPanel1.remove(board);
                    }
                    if (pic != null){
                        pic.flush();
                        pic = null;
                    }
                    board = new PlayingBoard(new PhraseScrambler(foo));
                    addPanel();
                    setTitle();
                    updateMenues();
                }
            }
        });
        newPatternMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                Picto foo = PatternDataSetter.getInstance().showMessage();
                if (foo != null){
                    if (board != null){
                        jPanel1.remove(board);
                    }
                    pic = foo;
                    board = new CreatePatternBoard(foo);
                    xSize = ((Playable)board).getXSize();
                    ySize = ((Playable)board).getYSize();
                    setTitle();
                    board.setVisible(true);
                    jPanel1.add(board);
                    centerPanel();
                    repaint();
                }
                updateMenues();
            }
        });
        loadGameMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                loadGame();
                updateMenues();
            }
        });
        saveGameMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                saveGame();
            }
        });
        quitMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                Config.saveConfig();
                System.exit(0);
            }
        });
        esMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                Config.getInstance().setLocale(Locales.SPANISH);
            }
        });
        enMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                Config.getInstance().setLocale(Locales.ENGLISH);
            }
        });
        clearMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                if (JOptionPane.showConfirmDialog
                   (jPanel1, _("Are you sure you want\nto clear the board?"),
                  _("Confirm"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION){
                    ((Playable)board).clean();
                }
            }
        });
        checkMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                ((Playable)board).checkBoard(false);
            }
        });
        checkAndResetMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                ((Playable)board).checkBoard(true);
            }
        });
        solveMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                ((Playable)board).solve();
            }
        });
        helpMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                if (helpWindow == null){
                    helpWindow = new HelpWindow();
                }
                helpWindow.setVisible(true);
            }
        });
        aboutMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                if (aboutWindow == null){
                    aboutWindow = new AboutWindow();
                }
                aboutWindow.setVisible(true);
            }
        });
        colorsMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                if (colorChooserWindow == null){
                    colorChooserWindow = new ColorChooser(MainWindow.this, false);
                }
                colorChooserWindow.setVisible(true);
            }
        });
        dragMenuItem.setSelected(Config.getInstance().isDraggingEnabled());
        dragMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                Config.getInstance().setDraggingEnabled(dragMenuItem.isSelected());
                ColoredButton.setDragEnabled(dragMenuItem.isSelected());
            }
        });
        mouseTrackerMenuItem.setSelected(Config.getInstance().isMouseTrackingEnabled());
        mouseTrackerMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                try{
                    Config.getInstance().setMouseTrackingEnabled(mouseTrackerMenuItem.isSelected());
                    if (board!=null) {
                        ((PlayingBoard) board).setTrakingEnabled(mouseTrackerMenuItem.isSelected());
                    }
                } catch (ClassCastException ex){}
            }
        });
        addWindowListener(new WindowAdapter() {
            @Override public void windowClosing(WindowEvent e) {
                Config.saveConfig();
                System.exit(0);
          }
        });
        addComponentListener(new ComponentAdapter() {
            @Override public void componentResized(ComponentEvent evt) {
                centerPanel();
            }
        });
        
    }
    private int kcode = 0;
    private void kcodeEntered(){
        JOptionPane.showMessageDialog(this, _("Cheater!"), _("AHA!")
                                    , JOptionPane.INFORMATION_MESSAGE);
        menuBar.add(extrasMenu, menuBar.getComponentIndex(helpMenu));
        extrasMenu.add(checkAndResetMenuItem);
        extrasMenu.add(solveMenuItem);
        menuBar.validate();
    }

    private void updateMenues(){
        checkAndResetMenuItem.setEnabled(board != null && board instanceof Playable);
        checkMenuItem.setEnabled(board != null && board instanceof Playable);
        solveMenuItem.setEnabled(board != null && board instanceof Playable);
        saveGameMenuItem.setEnabled(board != null);
        clearMenuItem.setEnabled(board != null);
    }

    private JMenuBar menuBar = new JMenuBar();
    private JMenu appMenu = new JMenu();
    private JMenuItem openRandomPhraseMenuItem = new JMenuItem();
    private JMenuItem openRandomGameMenuItem = new JMenuItem();
    private JMenuItem openPatternMenuItem = new JMenuItem();
    private JMenuItem loadGameMenuItem = new JMenuItem();
    private JMenuItem saveGameMenuItem = new JMenuItem();
    private JMenuItem quitMenuItem = new JMenuItem();
    private JMenu editMenu = new JMenu();
    private JMenu localeMenu = new JMenu();
    private JCheckBoxMenuItem enMenuItem = new JCheckBoxMenuItem();
    private JCheckBoxMenuItem esMenuItem = new JCheckBoxMenuItem();
    private ButtonGroup localeButtonGroup = new ButtonGroup();
    private JCheckBoxMenuItem dragMenuItem = new JCheckBoxMenuItem();
    private JCheckBoxMenuItem mouseTrackerMenuItem = new JCheckBoxMenuItem();
    private JMenuItem colorsMenuItem = new JMenuItem();
    private JMenu patternMenu = new JMenu();
    private JMenuItem clearMenuItem = new JMenuItem();
    private JMenuItem checkMenuItem = new JMenuItem();
    private JMenuItem newPatternMenuItem = new JMenuItem();
    private JMenuItem newPhraseMenuItem = new JMenuItem();
    private JMenu extrasMenu = new JMenu();
    private JMenuItem checkAndResetMenuItem = new JMenuItem();
    private JMenuItem solveMenuItem = new JMenuItem();
    private JMenu helpMenu = new JMenu("?");
    private JMenuItem helpMenuItem = new JMenuItem();
    private JMenuItem aboutMenuItem = new JMenuItem();
    private GameLoader gameLoader = new GameLoader(this);
    private AboutWindow aboutWindow;
    private HelpWindow helpWindow;
    private ColorChooser colorChooserWindow;

    private class ImagePanel extends JPanel {
        private static final long serialVersionUID = 7483567871L;
        private Image img;

        public ImagePanel(Image image) {
            setFocusable(true);
            img = new ImageIcon(MainWindow.class.getResource(
                                    "/res/images/dkt.png")).getImage();
            addKeyListener(new KeyAdapter() {
                @Override public void keyPressed(KeyEvent evt) {
                    if(kcode==-1||board==null||board instanceof CreatePatternBoard){
                        return;
                    }
                    int event = evt.getKeyCode();
                    if (event == KeyEvent.VK_UP){
                        kcode = (kcode==0||kcode==1) ? kcode + 1 : 0;
                    } else if(event == KeyEvent.VK_DOWN){
                        kcode = (kcode==2||kcode==3) ? kcode + 1 : 0;
                    } else if(event == KeyEvent.VK_LEFT){
                        kcode = (kcode==4||kcode==6) ? kcode + 1 : 0;
                    } else if(event == KeyEvent.VK_RIGHT){
                        kcode = (kcode==5||kcode==7) ? kcode + 1 : 0;
                    } else if(event == KeyEvent.VK_B){
                        kcode = (kcode==8) ? kcode + 1 : 0;
                    } else if(event == KeyEvent.VK_A){
                        if (kcode==9){
                            kcodeEntered();
                            kcode = -1;
                        }
                    } else {
                        kcode = 0;
                    }
                }
            });
        }

        @Override
        public void paintComponent(Graphics g) {
            g.setColor(Config.getInstance().getBackgroundColor());
            g.fillRect(0, 0, getWidth(), getHeight());
            if (board == null){
                int x = (getWidth() - img.getWidth(null)) / 2;
                int y = (getHeight() - img.getHeight(null)) / 2;
                g.drawImage(img, x, y, null);
            } else {
                img = null;
            }
        }

    }
}

