/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.interfaces;

/**
 * This class implements a panel in which a piece of code is going to be
 * shown, it adds the colors and everything
 *
 * @author Fido <dktcoding [at] gmail>
 */
public interface Localizable extends Locales{
    /**
     * This method is called in order to update all the localizable
     * texts in the application, so it should contain all the necessary
     * things to change the language without bothering the user
     * (i.e. no restarting, closing, etc.). The best way to accomplish this
     * is using a static import of the _ method in the text class:<code> static
     * import picto.utils.Text._;</code>
     * @see picto.utils.Text
     */
    public void updateTexts();

    /**
     * This method should be called from the constructor of the classes
     * that implement this interface, the idea is to create a weak reference
     * of every localizable object in order to change the localization from
     * all the available forms at once.<br>
     * The implementation of the method should be a single line like this one:
     * <pre>               <code> Text.addWeakReference(this);</code></pre>
     * The default package of this class is: <code>picto.utils.Text</code>
     * @see picto.utils.Text
     */
    public void addWeakReference();

}
