/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.utils;

import picto.app.interfaces.Status;
import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import picto.app.interfaces.Playable;
import picto.app.interfaces.Updatable;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class ColoredButton extends JButton implements Updatable{
    private static final long serialVersionUID = 56456565765324256L;
    private static boolean drag_enabled = Config.getInstance().isDraggingEnabled();
    private static boolean left_pressed = false;
    private static boolean right_pressed = false;
    private static Playable container = null;
    private byte curr_stat = Status.MMMM;
    private final byte ii;
    private final byte jj;

    public ColoredButton(byte i, byte j){
        super();
        this.ii = i;
        this.jj = j;
        setBorder(i, j);
        setFocusable(false);
        setBackground(Config.getInstance().getColorButtonBackgroundEmpty());
        setContentAreaFilled(false);
        setOpaque(true);
        addMouseListener(new MouseListener() {
            @Override
            public void mouseClicked(MouseEvent e) {
                if (!drag_enabled){
                    if (e.getButton() == MouseEvent.BUTTON1){
                        leftClickPressed();
                    } else if (e.getButton() == MouseEvent.BUTTON3){
                        rightClickPressed();
                    }
                }
            }
            @Override
            public void mousePressed(MouseEvent e) {
                if (drag_enabled){
                    if (e.getButton() == MouseEvent.BUTTON1){
                        if (!right_pressed){
                            left_pressed = true;
                            mouseEntered(e);
                        }
                    } else if (e.getButton() == MouseEvent.BUTTON3){
                        if (!left_pressed){
                            right_pressed = true;
                            mouseEntered(e);
                        }
                    }
                }
            }
            @Override
            public void mouseReleased(MouseEvent e) {
                if (drag_enabled){
                    left_pressed = right_pressed = false;
                }
            }
            @Override
            public void mouseEntered(MouseEvent e) {
                if (container != null){
                    container.updateHeaderFocus(ii, jj);
                }
                if (drag_enabled){
                    if (left_pressed){
                        leftClickPressed();
                    } else if (right_pressed){
                        rightClickPressed();
                    }
                }
            }
            @Override
            public void mouseExited(MouseEvent e) {
            }
        });
    }

    /**
     * Sets the border of the cell according to it's position, this way it knows
     * if the border must be double or single.
     *
     * @param i
     * @param j
     */
    private void setBorder(int i, int j){
        int top = 1, left = 1, bottom = 0, right = 0;
        if((i + 1) % 5 == 0) {
            bottom = 1;
        }
        if ((j + 1) % 5 == 0){
            right = 1;
        }

        setBorder(BorderFactory.createMatteBorder(top, left, bottom, right
                        , Config.getInstance().getColorButtonBorderColor()));
    }

    /**
     * Sets the background of the cell
     *
     * @param col color to be used
     */
    public void setColor(Color col){
        setBackground(col);
    }

    @Override
    public void update(){
        switch (curr_stat){
            case SOMETHING:
                setBackground(Config.getInstance().getColorButtonBackgroundSomething());
                break;
            case NOTHING:
                setBackground(Config.getInstance().getColorButtonBackgroundNothing());
                break;
            case MMMM:
                setBackground(Config.getInstance().getColorButtonBackgroundEmpty());
                break;
        }
    }

    public static void setContainer(Playable board){
        ColoredButton.container = board;
    }

    @Override
    public byte getStatus(){
        return curr_stat;
    }

    @Override
    public void setStatus(byte stat){
        curr_stat = stat;
        update();
    }

    /**
     * What to do if you do a left click
     */
    private void leftClickPressed(){
        if (curr_stat != Status.SOMETHING){
            curr_stat = Status.SOMETHING;
        } else {
            curr_stat = Status.MMMM;
        }
        update();
    }

    /**
     * What to do if you do a right click
     */
    private void rightClickPressed(){
        if (curr_stat != Status.NOTHING){
            curr_stat = Status.NOTHING;
        } else {
            curr_stat = Status.MMMM;
        }
        update();
    }

    @Override
    public void reset(){
        setBackground(Config.getInstance().getColorButtonBackgroundEmpty());
        curr_stat = Status.MMMM;
    }

    /**
     * Tells if mouse dragging is enabled
     *
     * @return <code>true</code> dragging is enabled and <code>false</code>
     * if it isn't
     */
    public static boolean isDragEnabled() {
        return drag_enabled;
    }

    public static void setDragEnabled(boolean drag_enabled) {
        ColoredButton.drag_enabled = drag_enabled;
    }

}
