/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.utils;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.logging.Level;
import java.util.logging.Logger;
import picto.app.interfaces.Locales;

/**
 * This singleton object contains all the configurable options of the game
 * 
 * @author Fido <dktcoding [at] gmail>
 */
public class Config implements Serializable, Locales{
    private static final long serialVersionUID = -85496494185623464L;
    private static Config instance;

    private Config(){

    }

    public synchronized static Config getInstance(){
        if (instance == null){
            if (!loadConfig()){
                instance = new Config();
            }
            // Yes I know what you're thinking! but 'instance' was
            // initialized in loadConfig()! JA!
        }
        return instance;
    }

    public void resetToDefaults(){
        instance = new Config();
    }

    public static Config getDefaultInstance(){
        return new Config();
    }

    private int locale = ENGLISH;
    public int getLocale(){
        return locale;
    }

    public void setLocale(int locale){
        this.locale = locale;
        Text.setLocale(locale);
    }

    private Color strikedTextForeground = Color.RED;
    public Color getStrikedTextForeground(){
        return strikedTextForeground;
    }

    public void setStrikedTextForeground(Color col){
        strikedTextForeground = col;
    }

    private Color headerFontForeground = Color.BLACK;
    public Color getHeaderFontForeground(){
        return headerFontForeground;
    }

    public void setHeaderFontForeground(Color col){
        headerFontForeground = col;
    }

    private Color headerBorderColor = Color.GRAY;
    public Color getHeaderBorderColor(){
        return headerBorderColor;
    }

    public void setHeaderBorderColor(Color col){
        headerBorderColor = col;
    }

    private Color headerBackgroundColor = Color.LIGHT_GRAY;
    public Color getHeaderBackgroundColor(){
        return headerBackgroundColor;
    }

    public void setHeaderBackgroundColor(Color col){
        headerBackgroundColor = col;
    }

    private Color headerBackgroundFocusColor = Color.CYAN;
    public Color getHeaderBackgroundFocusColor(){
        return headerBackgroundFocusColor;
    }

    public void setHeaderBackgroundFocusColor(Color col){
        headerBackgroundFocusColor = col;
    }

    private Font headerFont = new Font("DejaVu Sans", 0, 9);
    public Font getHeaderFont(){
        return headerFont;
    }

    public void setHeaderFont(Font font){
        headerFont = font;
    }
    private Color colorButtonBackgroundEmpty = Color.WHITE;
    public Color getColorButtonBackgroundEmpty(){
        return colorButtonBackgroundEmpty;
    }

    public void setColorButtonBackgroundEmpty(Color col){
        colorButtonBackgroundEmpty = col;
    }

    private Color colorButtonBackgroundNothing = Color.GREEN;
    public Color getColorButtonBackgroundNothing(){
        return colorButtonBackgroundNothing;
    }

    public void setColorButtonBackgroundNothing(Color col){
        colorButtonBackgroundNothing = col;
    }

    private Color colorButtonBackgroundSomething = Color.BLACK;
    public Color getColorButtonBackgroundSomething(){
        return colorButtonBackgroundSomething;
    }

    public void setColorButtonBackgroundSomething(Color col){
        colorButtonBackgroundSomething = col;
    }

    private Color colorButtonBorderColor = Color.GRAY;
    public Color getColorButtonBorderColor(){
        return colorButtonBorderColor;
    }

    public void setColorButtonBorderColor(Color col){
        colorButtonBorderColor = col;
    }

    private Color backgroundColor = new Color(-0x666601);
    public Color getBackgroundColor(){
        return backgroundColor;
    }

    public void setBackgroundColor(Color col){
        backgroundColor = col;
    }
    
    private boolean draggingEnabled = true;
    public boolean isDraggingEnabled(){
        return draggingEnabled;
    }

    public void setDraggingEnabled(boolean dragging_enabled){
        draggingEnabled = dragging_enabled;
    }

    private boolean mouseTrackingEnabled = false;
    public boolean isMouseTrackingEnabled(){
        return mouseTrackingEnabled;
    }

    public void setMouseTrackingEnabled(boolean mouse_tracking_enabled){
        mouseTrackingEnabled = mouse_tracking_enabled;
    }

    /**
     * @return Application name
     */
    public static String getAppName(){
        return "PictoGame...";
    }

    /**
     * @return Application description
     */
    public static String getAppDescription(){
        return "A simple game for playing and creating nonograms";
    }

    /**
     * @return URL of this application
     */
    public static String getAppURL(){
        return "http://launchpad.net/pictogame";
    }

    /**
     * @return Version number
     */
    public static String getAppVersion(){
        return "0.0.3";
    }

    /**
     * @return Author
     */
    public static String getAppAuthor(){
        return "Fido";
    }

    /**
     * @return Author's Email
     */
    public static String getAppAuthorEmail(){
        return "<dktcoding [at] gmail>";
    }

    /**DKT's application's path*/
    private final static String dktpath = System.getProperty("user.home")+"/.dkt";
    /**Game path*/
    private final static String minespath = "/pictogame";
    /**Name of the file*/
    private final static File file = new File(dktpath+minespath+"/pictogame.config");

    private static boolean all_ready_checked = false;
    /**
     * Checks if the directory exists if it doesn't exist then it's created
     */
    private static void checkAndCreateDirectory(){
        if (all_ready_checked){
            return;
        }
        if (!file.exists()){
            if (!new File (dktpath).exists()){
                new File (dktpath).mkdir();
            }
            if (!new File (dktpath+minespath).exists()){
                new File (dktpath+minespath).mkdir();
                all_ready_checked = true;
            }
        }
    }

    /**
     * Saves the current configuration
     */
    public static void saveConfig(){
        try {
            checkAndCreateDirectory();
            ObjectOutputStream oos = null;
            oos = new ObjectOutputStream(new FileOutputStream(file));
            oos.writeObject(instance);
            oos.flush();
            oos.close();
        } catch (IOException ex) {
            Logger.getLogger(Config.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * Load the saved config
     * @return success
     */
    private static boolean loadConfig(){
        Boolean state = false;
        try {
            checkAndCreateDirectory();
            ObjectInputStream ois = null;
            ois = new ObjectInputStream(new FileInputStream(file));
            instance = (Config) ois.readObject();
            ois.close();
            state = true;
        } catch (FileNotFoundException ex) {
            System.err.println("No config file found setting default values");
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(Config.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(Config.class.getName()).log(Level.SEVERE, null, ex);
        }
        return state;
    }
}
