/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.utils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import picto.app.interfaces.Locales;
import picto.app.interfaces.Localizable;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class PhraseScrambler implements Serializable, Localizable{
    private static final long serialVersionUID = -5670613584791023L;
    private static PhraseScrambler instance = new PhraseScrambler();
    private static Random rand = new Random(System.nanoTime());
    private static ArrayList<String> list;
    private char[][] obfuscator;
    private String phrase;
    private Picto pic;

    private PhraseScrambler(){
        updateTexts();
        addWeakReference();
    }
    
    public PhraseScrambler (String phrase){
        this.phrase = phrase;
        // The following steps (scramble and obfuscate) can be easily set in
        // just one loop, but I believe is clearer this way, specially
        // considering that we want to obfuscate the phrase and not the code
        scramble();
        obfuscate();
    }
    
    public void generate(String phrase){
        
    }

    public final void scramble(){
        int length = phrase.split("--")[0].replace(" ", "").length();
        int ySize, xSize;
        {// The following is based on solid ... err ... "facts", I demand that I
         // may ... or may not be Vroomfondel
            ySize = 3 + length / 12;
            xSize = 5 + length / 13 + length / ySize;
            int r = rand.nextInt(4);
            xSize -= r;
            ySize += r;
        }
        int counter = xSize * ySize - length;
        pic = new Picto(ySize, xSize);
        int i,j;
        do{
            i = rand.nextInt(ySize);
            j = rand.nextInt(xSize);
            if (!pic.getElement(i, j)){
                pic.setElement(i, j, true);
                counter--;
            }
        } while (counter != 0);
        pic.regenerate();
    }

    public final void obfuscate(){
        obfuscator = new char[pic.getNumberOfRows()][pic.getNumberOfColumns()];
        String foo = phrase.split("--")[0].replace(" ", "").toUpperCase();
        int counter = foo.length();
        for (int i = pic.getNumberOfRows()-1; i >= 0 ; i--){
            for (int j = pic.getNumberOfColumns()-1; j >= 0 ; j--){
                if (pic.getElement(i, j)){
                    // Get a random upper case char
                    // (char)(rand.nextInt((int)'Z'-(int)'A')+'A');
                    obfuscator[i][j] = (char)(rand.nextInt(25) + 65);
                } else {
                    obfuscator[i][j] = foo.charAt(--counter);
                }
            }
        }
    }

    public static void main (String ... args){
        System.out.println(PhraseScrambler.getRandomScrambler());
    }

    @Override
    public String toString(){
        String str = pic.toString();
        str += "\n";
        for (int i = 0; i < pic.getNumberOfRows(); i++){
            for (int j = 0; j < pic.getNumberOfColumns(); j++){
                str += obfuscator[i][j];
            }
            str += "\n";
        }
        str += "\n";
        for (int i = 0; i < pic.getNumberOfRows(); i++){
            for (int j = 0; j < pic.getNumberOfColumns(); j++){
                if (pic.getElement(i, j)){
                    str += " ";
                } else {
                    str += obfuscator[i][j];
                }
            }
            str += "\n";
        }
        return str;
    }

    public String getPhrase() {
        return phrase;
    }

    public void setPhrase(String phrase) {
        this.phrase = phrase;
    }

    public Picto getPic() {
        return pic;
    }

    public void setPic(Picto pic) {
        this.pic = pic;
    }

    public char getElement(int i, int j){
        return obfuscator[i][j];
    }

    public static PhraseScrambler getRandomScrambler(){
        if (list.isEmpty()){
            instance.updateTexts();
        }
        return new PhraseScrambler(list.remove(rand.nextInt(list.size())));
    }

    @Override
    public final void updateTexts() {
        list = new ArrayList<String>(50);
        String lang = Locales.locales[Text.getLocale()].getISO3Language();
        BufferedReader reader = new BufferedReader(new InputStreamReader(
                 PhraseScrambler.class.getResourceAsStream("/res/phrases/"+lang)));
        String line = null;
        try {
            line = reader.readLine();
            while (line != null){
                list.add(line);
                line = reader.readLine();
            }
            reader.close();
        } catch (IOException ex) {
            Logger.getLogger(PhraseScrambler.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    @Override
    public final void addWeakReference() {
        Text.addWeakReference(this);
    }
}
