/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.utils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import picto.app.interfaces.Localizable;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class SavingUtils implements Localizable {
    private static SavingUtils instance = new SavingUtils();
    private static String ERROR_MSG1;

    /**
     * Saves the current game
     *
     * @param file where to save the game
     * @param saver game saver
     * @see picto.utils.Saver
     */
    public static void saveGame(File file, Saver saver){
        if (!file.getPath().endsWith(".PictoSave")){
            JOptionPane.showMessageDialog(null, _("Invalid file name!")
                                      , _("Error"), JOptionPane.ERROR_MESSAGE);
        } else {
            try {
                ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(file));
                oos.writeObject(saver);
                oos.flush();
                oos.close();
            } catch (IOException ex) {
                JOptionPane.showMessageDialog(null, _(ERROR_MSG1)
                                        , _("Error"), JOptionPane.ERROR_MESSAGE);
                Logger.getLogger(SavingUtils.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    /**
     * Saves a new pattern
     *
     * @param file where to save
     * @param pic the Picto to save
     * @see picto.app.Picto
     */
    public static void saveNewPattern(File file, Picto pic){
        if (!file.getPath().endsWith(".pattern")){
            JOptionPane.showMessageDialog(null, _("Invalid file name!")
                                    , _("Error"), JOptionPane.ERROR_MESSAGE);
        } else {
            try {
                FileWriter filew = new FileWriter(file);
                filew.write(pic.toSavingString());
                filew.flush();
                filew.close();
            } catch (IOException ex) {
                JOptionPane.showMessageDialog(null, _(ERROR_MSG1)
                                        , _("Error"), JOptionPane.ERROR_MESSAGE);
                Logger.getLogger(SavingUtils.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    private SavingUtils() {
        updateTexts();
        addWeakReference();
    }

    @Override
    public final void updateTexts() {
        ERROR_MSG1 = _("Error while writing the file!\n"
                     + "Check if you have writing access to the medium");
    }

    @Override
    public final void addWeakReference() {
        Text.addWeakReference(instance);
    }

}
