/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.utils;

import picto.app.interfaces.Localizable;
import picto.app.interfaces.Locales;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class Text implements Locales{
    private static ArrayList<WeakLocalizable> open_localizables
                                 = new ArrayList<WeakLocalizable>(20);
    private final static String pathStr = "/res/i18n/strings";
    private static Properties LOCAL_TEXT = new Properties();
    private static int current_locale = -1;

    static{setLocale(Config.getInstance().getLocale());}

    private Text(){

    }

    public static int getLocale(){
        return current_locale;
    }

    public static void setLocale(int new_locale){
        if (current_locale == new_locale){
            return;
        }
        LOCAL_TEXT.clear();
        JFileChooser.setDefaultLocale(locales[new_locale]);
        JOptionPane.setDefaultLocale(locales[new_locale]);
        try {
            String ID = "";
            if (new_locale != ENGLISH){
                ID = "_" + locales[new_locale].getLanguage();
            }
            ID += ".properties";
            LOCAL_TEXT.load(new InputStreamReader(Text.class
                                    .getResourceAsStream(pathStr + ID)));
        } catch (IOException ex) {
            Logger.getLogger(Text.class.getName()).log(Level.SEVERE, null, ex);
        }
        current_locale = new_locale;
        update();
    }

    public static void addWeakReference(Localizable e){
        open_localizables.add(new WeakLocalizable(e));
    }

    private static void update(){
        for (int i = 0; i < open_localizables.size(); i++){
            if (open_localizables.get(i).get() != null){
                open_localizables.get(i).get().updateTexts();
            } else {
              open_localizables.remove(i--);
            }
        }
    }

    /**
     * Method that returns the string to be displayed, if no string is found
     * then it falls back to English
     *
     * @param msg
     * @return Localized string
     */
    public synchronized static String _(String msg) {
        String text = LOCAL_TEXT.getProperty(msg);
        return (text == null) ? msg : text;
    }

    private static class WeakLocalizable extends WeakReference<Localizable>{
        private WeakLocalizable(Localizable referent){
            super(referent);
        }
    }
}
