/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) DKT 2010 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.utils;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import picto.app.interfaces.Focusable;
import picto.app.interfaces.Updatable;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class TextButton extends JButton implements Updatable, Focusable{
    private static final long serialVersionUID = 56456565324256L;
    private static Config config = Config.getInstance();
    private boolean pressed = false;
    private final Byte number;
    private boolean is_column;

    public TextButton(int num, boolean is_column, byte i){
        super();
        this.is_column = is_column;
        this.number = (byte)num;
        setFocusable(false);
        setContentAreaFilled(false);
        setOpaque(true);
        setBackground(config.getHeaderBackgroundColor());
        setBorder(i);
        if (number != 0){
            setForeground(config.getHeaderFontForeground());
            setFont(config.getHeaderFont());
            setText(number.toString());
            addActionListener(new ActionListener() {
                @Override public void actionPerformed(ActionEvent e) {
                    pressed = !pressed;
                    if (pressed){
                        setForeground(config.getStrikedTextForeground());
                        setText(start+number+end);
                    } else {
                        reset();
                    }
                }
            });
        }
    }
    private static String start = "<html><strike>";
    private static String end = "</strike></html>";

    @Override
    public void reset(){
        if (number != 0){
            setForeground(config.getHeaderFontForeground());
            setText(number.toString());
        }
    }

    private void setBorder(byte i){
        byte top = 1, left = 1, bottom = 0, right = 0;
        if (is_column){
            top = bottom = 0;
            if (i == 0 || (i + 1) % 5 == 1){
                left = 1;
            } else if((i + 1) % 5 == 0) {
                right = 1;
            }
        } else {
            left = right = 0;
            if (i == 0 || (i + 1) % 5 == 1){
                top = 1;
            } else if((i + 1) % 5 == 0) {
                bottom = 1;
            }
        }
        setBorder(BorderFactory.createMatteBorder(top, left, bottom, right
                                        , config.getHeaderBorderColor()));
    }

    @Override
    public void giveFocus(){
        if (number != 0){
            setBackground(config.getHeaderBackgroundFocusColor());
        }
    }

    @Override
    public void removeFocus(){
        if (number != 0){
            setBackground(config.getHeaderBackgroundColor());
        }
    }

    @Override
    public void setStatus(byte stat){
        if (stat == TEXT_CROSSED){
            pressed = true;
            setForeground(config.getStrikedTextForeground());
            setText("<html><strike>"+number+"</strike></html>");
        }
    }

    @Override
    public byte getStatus(){
        return (number == 0) ? -1 : pressed ? TEXT_CROSSED : TEXT_NORMAL;
    }

    @Override
    public void update() {
        setBackground(config.getHeaderBackgroundColor());
        if (number == 0){return;}
        if (getStatus() == TEXT_CROSSED){
            setForeground(config.getStrikedTextForeground());
            setFont(config.getHeaderFont());
        } else {
            setForeground(config.getHeaderFontForeground());
            setFont(config.getHeaderFont());
        }
    }

}
